/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/core-module.h"
#include "ns3/simulator-module.h"
#include "ns3/node-module.h"
#include "ns3/helper-module.h"
#include "ns3/leaky-bucket-helper.h"
#include "ns3/flow-monitor-module.h"

// Default Network Topology
//
//       10.1.1.0
// n0 -------------- n1   n2   n3   n4
//    point-to-point  |    |    |    |
//                    ================
//                      LAN 10.1.2.0


using namespace ns3;

NS_LOG_COMPONENT_DEFINE ("SecondScriptExample");

int 
main (int argc, char *argv[])
{
  NodeContainer p2pNodes;
  p2pNodes.Create (2);

  NodeContainer csmaNodes;
  csmaNodes.Add (p2pNodes.Get (1));
  csmaNodes.Create (3);

  PointToPointHelper pointToPoint;
  pointToPoint.SetDeviceAttribute ("DataRate", StringValue ("10Mbps"));
  //pointToPoint.SetChannelAttribute ("Delay", StringValue ("2ms"));

  NetDeviceContainer p2pDevices;
  p2pDevices = pointToPoint.Install (p2pNodes);

  CsmaHelper csma;
  csma.SetChannelAttribute ("DataRate", StringValue ("100Mbps"));
  csma.SetChannelAttribute ("Delay", TimeValue (NanoSeconds (6560)));

  NetDeviceContainer csmaDevices;
  csmaDevices = csma.Install (csmaNodes);

  InternetStackHelper stack;
  stack.Install (p2pNodes.Get (0));
  stack.Install (csmaNodes);

  Ipv4AddressHelper address;
  address.SetBase ("10.1.1.0", "255.255.255.0");
  Ipv4InterfaceContainer p2pInterfaces;
  p2pInterfaces = address.Assign (p2pDevices);

  address.SetBase ("10.1.2.0", "255.255.255.0");
  Ipv4InterfaceContainer csmaInterfaces;
  csmaInterfaces = address.Assign (csmaDevices);

   ApplicationContainer apps0;
  OnOffHelper onOffHelper0  ("ns3::UdpSocketFactory", InetSocketAddress (Ipv4Address ("10.1.2.4"), 9));      
  onOffHelper0.SetAttribute ("DataRate", StringValue ("10Mbps"));                                           
  onOffHelper0.SetAttribute ("PacketSize", UintegerValue (1));      
  onOffHelper0.SetAttribute ("MaxBytes", UintegerValue (10));                                          
  onOffHelper0.SetAttribute ("OnTime",  RandomVariableValue (ConstantVariable (1)));                        
  onOffHelper0.SetAttribute ("OffTime", RandomVariableValue (ConstantVariable (0)));                     
  apps0.Add(onOffHelper0.Install (p2pNodes.Get (0)));      


/////////////////////////
 LeakyBucketHelper myBucket;
  myBucket.SetAttribute ("DataRate", StringValue ("1Mbps")); 
  myBucket.SetAttribute ("Ipv4Shape", Ipv4AddressValue ("10.1.1.1")); 
  myBucket.SetAttribute ("MaxPackets", UintegerValue (5)); 
  myBucket.Install(p2pDevices.Get(1));
/////////////////////////

  Ipv4GlobalRoutingHelper::PopulateRoutingTables ();

 /*========================================FlowMonitorHelper=================================================*/
  FlowMonitorHelper flowmon;
  Ptr<FlowMonitor> monitor = flowmon.InstallAll();

  Simulator::Stop (Seconds (30.0));
  Simulator::Run ();
/*================================Print per flow statistics=================================================*/
  monitor->CheckForLostPackets ();
  Ptr<Ipv4FlowClassifier> classifier = DynamicCast<Ipv4FlowClassifier> (flowmon.GetClassifier ());
  std::map<FlowId, FlowMonitor::FlowStats> stats = monitor->GetFlowStats ();
  for (std::map<FlowId, FlowMonitor::FlowStats>::const_iterator i = stats.begin (); i != stats.end (); ++i)
    {
      Ipv4FlowClassifier::FiveTuple t = classifier->FindFlow (i->first);
      std::cout << "Flow " << i->first << " (" << t.sourceAddress << " -> " << t.destinationAddress << ")\n";
      std::cout << "  Tx Bytes:   " << i->second.txBytes << "\n";
      std::cout << "  Rx Bytes:   " << i->second.rxBytes << "\n";
      std::cout << "  Tx Packets:  " << i->second.txPackets << "\n";
      std::cout << "  Rx Packets:  " << i->second.rxPackets << "\n";
      std::cout << "  delaySum:  " << i->second.delaySum << "\n"; 
    }

 Simulator::Destroy ();
  return 0;
}
